#!/usr/bin/perl

use Gimp qw( :auto);
use strict;

package Bfe::Graphics;

sub make_printer_food {

  my ($infile, $outfile, $scale) = @_;

  $scale = 1.0 unless (defined $scale);

  my $img = Gimp::gimp_file_load(1, $infile, $infile);
  my $layer = $img->active_drawable;
  $img->scale($scale*$img->width, $scale*$img->height) unless ($scale == 1.0);
  $layer->fsdither(2, 1, 1, 1, 1, 1);
  Gimp::file_print(1, $img, $layer, $outfile,
		   "pcl-5",	# printer driver	
		   "",
		   0,		# gray output
		   "600",	# dpi
		   "Letter",	# paper type
		   "Plain",
		   "Manual",
		   100,
		   100,
		   1,
		   -1,
		   -1);
  $img->delete;
}


sub make_bar {			# makes <hr> substitute

  my $list = shift;

  foreach my $image (@{$list}) {
    unless (-f $image->{name}) {
      my ($name, $width, $height, $pattern, $offset, $bump_params) = 
	($image->{name}, $image->{width}, $image->{height}, 
	 $image->{pattern}, $image->{offset}, $image->{bump_params}); 
     
      $bump_params = [135,	# azimuth
		      35,	# elevation
		      10,	# depth
		      0,	# xoffset
		      0,	# yoffset
		      50,	# waterlevel
		      0,	# ambient
		      1,	# compesenate for darkening
		      0,	# invert
		      0] unless (defined $bump_params); # linear map     

      my $img = Gimp::gimp_image_new(1, $width+$height+$offset*5, 
				     $height+$offset*10, 'RGBA'); 
      my $layer = Gimp::gimp_layer_new($img, $width+$height*$offset*5,
				       $height+$offset*10,
				      'RGBA', "bar", 100,
				      'NORMAL');
      $layer->add_alpha();
      $img->add_layer($layer, 0);
      $layer->fill(2); # set to alpha
      Gimp::gimp_palette_set_foreground([255, 255, 255]); 
      Gimp::gimp_ellipse_select($img, $height/2, $height/2, $height, $height,
			  'ADD', 1, 0, 0);
      Gimp::gimp_ellipse_select($img, $height/2+$width, $height/2, $height,
			  $height, 'ADD', 1, 0, 0);
      Gimp::gimp_rect_select($img, $height, $height/2, $width, $height, 'ADD', 0, 0);
      Gimp::gimp_edit_fill($img, $layer);
      Gimp::gimp_selection_clear($img);
      $layer = bump_map_and_drop_shadow($img, $layer, $pattern,
					$offset, 2*$offset, $bump_params);
      my $pwd = `pwd`;
      chop $pwd;
      save_image($img, $layer, $pwd.'/'.$image->{name});
      $img->delete;
    }
  }
}

sub save_image {

  my ($img, $layer, $filename) = @_;

  if($filename =~ /jpeg$/) {
    Gimp::file_jpeg_save(1, $img, $layer, $filename, $filename, 0.8, 0, 1);
  } else {
    Gimp::gimp_file_save(1, $img, $layer, $filename, $filename);
  }


}

sub make_text {			# would be nice if we could call perl fu 
				# from here
  my $list = shift;

  foreach my $image (@{$list}) {
    unless (-f $image->{name}) {
      
#      print "$image->{font} $image->{text} $image->{size} $image->{pattern}\n";

      my ($font, $text, $size, $pattern, $offset, $bump_params) = 
	($image->{font},$image->{text},$image->{size},
	 $image->{pattern}, $image->{shadow_height}, $image->{bump_params});

      die "font not defined\n" unless (defined $font);
      die "text not defined\n" unless (defined $text);
      die "size not defined\n" unless (defined $size);
      die "pattern not defined\n" unless (defined $pattern);
  
      $bump_params = [135,	# azimuth
		      35,	# elevation
		      3,	# depth
		      0,	# xoffset
		      0,	# yoffset
		      50,	# waterlevel
		      0,	# ambient
		      1,	# compesenate for darkening
		      0,	# invert
		      0] unless ($bump_params);	# linear map

      $offset = int($size/10) unless (defined $offset);

      # Create a new image of arbitrary size 
      my $img = Gimp::gimp_image_new(1, 256, 256, Gimp::RGB);
      
      # Choose color of text 
      Gimp::gimp_palette_set_foreground([255, 255, 255]); 
      
      # Create the text 
      my $text_layer = Gimp::gimp_text($img, -1, 0, 0, $text, 
				       int($offset+$size/6), 1, $size, 
				       'PIXELS', "*", $font, 
				       "*","*","*","*"); 

      my $width = $text_layer->width; 
      my $height = $text_layer->height; 
      
      $img->resize($width, $height, 0, 0); 
      
      my $layer = bump_map_and_drop_shadow($img, $text_layer, $pattern,
					   $offset, int($size/6), $bump_params);
      my $pwd = `pwd`;
      chop $pwd;
      save_image($img, $layer, $pwd.'/'.$image->{name});
      $img->delete;
    }
  }
}

sub bump_map_and_drop_shadow {

  my ($img, $layer, $pattern, $offset, $blur, $bump_params) = @_;
  
  my $bump_map = $layer->copy(0);

  $img->add_layer($bump_map, 1);

  Gimp::gimp_patterns_set_pattern(1, $pattern); 
  Gimp::gimp_selection_layer_alpha($img, $layer);
  Gimp::gimp_bucket_fill($img, $layer, 2, 0, 100, 0, 0, 0, 0);
  Gimp::gimp_selection_clear($img);
  
  $bump_map->gauss_rle($blur, 1, 1);
  $layer->bump_map($bump_map, @$bump_params);
  
  Gimp::gimp_invert($img, $bump_map);
  Gimp::gimp_channel_ops_offset($img, $bump_map, 0, 1, $offset, $offset);
  
  # Create a new layer for the background and fill it 
  # with a nice color, and add it to the image 

  my $width = $layer->width; 
  my $height = $layer->height; 
      
  my $background = Gimp::gimp_layer_new($img, $width, $height, 
					'RGB', "Background", 100, 
					'NORMAL'); 
  Gimp::gimp_palette_set_background([255, 255, 255]); 
  Gimp::gimp_edit_fill($img, $background); 
  $img->add_layer($background, 2); 
  my $new_layer = $img->flatten;

  $new_layer->zealouscrop;

  return $new_layer;
}

sub boxify_and_jpeg {		# draws inverted box on image and saves as jpeg
  my ($file, $x, $y, $width, $height, $thickness, $box) = @_; 
  
  my $img = Gimp::gimp_file_load(1, $file, $file);
  my $layer = $img->active_drawable();
  if(0) {
    if ($box) {
      Gimp::gimp_rect_select($img, $x, $y, $width, $height, 0, 0, 0);
      Gimp::gimp_rect_select($img, $x+$thickness, $y+$thickness,
			     $width - 2*$thickness,
			     $height - 2*$thickness,
			     1, 0, 0);
    } else {
      Gimp::gimp_rect_select($img, $x, $y, 1, 1, 0, 0, 0);
    }
    Gimp::gimp_invert($img, $layer);
    Gimp::gimp_selection_clear($img);
  }
  $file =~ s/\.png/\.jpeg/;
  Gimp::file_jpeg_save(1, $img, $layer, $file, $file, 0.9, 0, 0);
  $img->delete;
  return $img;			 # not necessary
} 
1;
